;; Generated package description from casual.el  -*- mode: lisp-data; no-byte-compile: t; lexical-binding:t -*-
(define-package "casual" "2.15.0.0.20260406.163831" "Transient user interfaces for various modes" '((emacs "30.1") (transient "0.9.0") (csv-mode "1.27")) :commit "81e032d24c46159cc98ec5248e48ee3659025ad0" :authors '(("Charles Choi" . "kickingvegas@gmail.com")) :maintainer '("Charles Choi" . "kickingvegas@gmail.com") :keywords '("tools" "wp") :url "https://github.com/kickingvegas/casual")
